# -*- coding: utf-8 -*-
"""
Created on Mon Nov 11 15:41:02 2024

@author: Giovanni
"""

import os
import sys
import numpy as np
import matplotlib.pyplot as plt
# Import the PCA reconstruction function from the specified folder
from PCA_reconstruction import PCA_reconstruction
# Load true signal
true_signal = np.loadtxt('validation.dat')
N, n = true_signal.shape

# Define bandwidth values to be tested
h = [0.5, 0.6, 0.8, 0.9, 0.95, 0.96, 0.97, 0.98, 0.99, 0.995, 1.0]
nn = len(h)

# Initialize lists to store results
n_PC = []
rmse = []
robustness = []
acc = []

# Turn off interactive plotting
plt.ioff()

# Loop through each bandwidth value
for i in range(nn):
    print(f"Iteration {i+1}/{nn} for var_perc = {h[i]}")

    # Perform PCA reconstruction on 'validation.dat'
    test_reconstruction_n, n_pc, rmse_i = PCA_reconstruction('train.dat', 'validation.dat', h[i])
    n_PC.append(n_pc)
    print(f'number of principal components: {n_PC}')
    rmse.append(rmse_i)
    
    # Perform PCA reconstruction on 'validation_sim.dat'
    test_reconstruction, n_pcx, no_int = PCA_reconstruction('train.dat', 'validation_sim.dat', h[i])

    # Calculate robustness and accuracy
    robustness_val = np.sqrt(np.sum((test_reconstruction[:, 2] - true_signal[:, 2]) ** 2)) / N
    acc_val = np.sqrt(np.sum((test_reconstruction_n[:, 2] - true_signal[:, 2]) ** 2)) / N
    robustness.append(robustness_val)
    acc.append(acc_val)

    # Close all figures generated in each iteration
    plt.close('all')

# Turn on interactive plotting
plt.ion()

# Plot results
plt.figure()
plt.plot(n_PC, rmse, '-*', label='RMSE')
plt.plot(n_PC, robustness, '-o', label='Robustness')
plt.xlabel('Number of PCs')
plt.legend()
plt.show()

plt.figure()
plt.plot(h, n_PC, '-*')
plt.xlabel('Variance Described')
plt.ylabel('Number of PCs')
plt.show()
